package dev.fitko.fitconnect.core.destination;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fitko.fitconnect.api.config.ApplicationConfig;
import dev.fitko.fitconnect.api.domain.auth.OAuthToken;
import dev.fitko.fitconnect.api.domain.model.destination.Destination;
import dev.fitko.fitconnect.api.domain.model.jwk.ApiJwk;
import dev.fitko.fitconnect.api.services.auth.OAuthService;
import dev.fitko.fitconnect.api.services.destination.DestinationService;
import dev.fitko.fitconnect.core.RestEndpointBase;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.http.MediaType;
import org.springframework.web.client.RestTemplate;

import java.util.UUID;

import static com.github.tomakehurst.wiremock.client.WireMock.*;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

class DestinationApiServiceTest extends RestEndpointBase {

    private DestinationService underTest;
    private OAuthService authServiceMock;

    @BeforeEach
    void setup() {
        final ApplicationConfig config = getTestConfig("http://localhost:" + wireMockServer.port());
        this.authServiceMock = mock(OAuthService.class);
        this.underTest = new DestinationApiService(this.authServiceMock, new RestTemplate(), config);
    }

    @Test
    void getDestination() throws JsonProcessingException {

        //Given
        final Destination expectedDestination = new Destination();
        expectedDestination.setDestinationId(UUID.randomUUID());

        final OAuthToken oAuthToken = new OAuthToken();
        oAuthToken.setAccessToken("123");
        when(this.authServiceMock.getCurrentToken()).thenReturn(oAuthToken);

        final var urlPath = "/v1/destinations/" + expectedDestination.getDestinationId();
        stubFor(urlPath, expectedDestination, MediaType.APPLICATION_JSON_VALUE);

        // When
        final Destination destination = this.underTest.getDestination(expectedDestination.getDestinationId());

        // Then
        assertEquals(expectedDestination, destination);
        verify(getRequestedFor(urlEqualTo(urlPath)).withHeader("Authorization", equalTo("Bearer 123")));

    }

    @Test
    void getEncryptionKey() throws Exception {
        // Given
        final UUID destinationId = UUID.randomUUID();
        final ApiJwk jwk = new ApiJwk();
        jwk.setKid(UUID.randomUUID().toString());

        final OAuthToken oAuthToken = new OAuthToken();
        oAuthToken.setAccessToken("123");
        when(this.authServiceMock.getCurrentToken()).thenReturn(oAuthToken);

        final var urlPath = "/v1/destinations/" + destinationId + "/keys/" + jwk.getKid();
        stubFor(urlPath, jwk, MediaType.APPLICATION_JSON_VALUE);

        // When
        final ApiJwk publicKey = this.underTest.getPublicKey(destinationId, jwk.getKid());

        // Then
        assertEquals(jwk, publicKey);
        verify(getRequestedFor(urlEqualTo(urlPath)).withHeader("Authorization", equalTo("Bearer 123")));
    }

    private void stubFor(final String url, final Object responseBody, final String contentType) throws JsonProcessingException {
        wireMockServer.stubFor(
                get(urlEqualTo(url))
                        .willReturn(ok()
                                .withBody(new ObjectMapper().writeValueAsString(responseBody))
                                .withHeader("Content-Type", contentType)
                                .withStatus(200)));
    }

}